#!/usr/bin/env python
# -*- coding: utf-8 -*-
#.--. .-. ... .... -. - ... .-.-.- .. -.

"""
metaRNA finds potential target sites for the microRNAs in genomic sequences.
To use metaRNA use explicit imports.

Example:
>>> from metarna.target_scan import scan, free_energy
>>> scan(**args)
"""



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#.--. .-. ... .... -. - ... .-.-.- .. -.

import json
from metarna.pymiranda import find_targets

def scan(target_sequence, mirna_sequence, **kwa):
  """
  Find targets of miRNA on the target based on the mirna and target sequences.

  Args:
    target_sequence (str): The target genomic sequence.
    mirna_sequence (str): The microRNA sequence.

  Optional Args: These parameters are passed on to the pymiranda algorithm.
    scale (float): The 5' miRNA scaling parameter. (default: 4.0)
    strict (int): Perform a Strict Seed search when set to 1. (default: 0)
    gap_open (float): Gap-open Penalty (default: -9.0)
    gap_extend (float): Gap-extend Penalty (default: -4.0)
    score_threshold (float): Score Threshold for reporting hits (default: 50.0)
    energy_threshold (float): Energy Threshold for reporting hits (default: 1.0)
    length_5p_for_weighting (int): The 5' sequence length to be weighed except
                                   for the last residue (default: 8).
    temperature (int): Used while calculating Free Energy (default: 30)
    alignment_len_threshold (int): Minimum alignment. (default: 8)

  Return (dict): A dictionary containing the stats of each possible target
                 site and the pairing obtained.
  """
  mirna_sequence_rev = mirna_sequence[::-1]
  enc_result = find_targets(target_sequence, mirna_sequence_rev, **kwa)
  return json.loads(enc_result)

def free_energy(target_sequence, mirna_sequence, **kwa):
  """
  Shortcut to find the Maximum Free energy in the given pair.

  See metarna.pymiranda.target_scan.scan for details on the arguments.

  Return (float): The free energy value.
  """
  try:
    doc = scan(target_sequence, mirna_sequence, **kwa)
    dat = max(doc.get('hits', []), key= lambda x: x['score'])
    return dat['energy']
  except (KeyError, ValueError):
    raise ValueError("No targets found.")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# metaRNA
'''Target Scan Report Generation

Each hits contain the following three attributes:
 - aln_mirna
 - aln_map
 - aln_utr
Where aln_utr is the sequence from the cDNA, aln_map are the base pairs, and
aln_mirna is the miRNA sequence.

Expected Render:
  acgguaacucaagguuuUAGCGGu
                   ||:|:|
  -------acaagatgccATTGTCc
'''

HIT_TEMPLATE = '''

  ΔG    {energy}

{query_end:>6}
     {aln_mirna}
     {aln_map}
     {aln_utr}
{ref_end:>6}   {ref_start}

'''

def get_report(targets):
  hits = targets.get('hits', [])
  hits.sort(key=lambda x: x['score'], reverse=True)
  out = ""
  for hit in hits:
    out += HIT_TEMPLATE.format(**hit)
  return out



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#.--. .-. ... .... -. - ... .-.-.- .. -.

import sys
import os
import platform

from setuptools import setup, Extension

def check_rna_lib_installed(**kwa):
  "Check Vienna RNA Packages"
  return all([os.path.exists(*v) for k, v in kwa.items()])

def get_rna_lib_intallation_path():
  "Attempts to get the RNAlib Path"
  return {
    'include_dirs': [
      os.environ.get('VIENNARNAINCLUDE', '/usr/local/include/ViennaRNA'),
    ],
    'library_dirs': [
      os.environ.get('VIENNARNALIB', '/usr/local/lib')
    ]
  }

def check_sanity():
  """
  Check Sanity of the System. Since metaRNA core depends on C extension,
  only posix systems are supported at the moment.
  """

  if not os.name == 'posix':
    print("metaRNA is currently not tested on non-posix systems.")

  if sys.version_info[:2] not in [(2, 7), (3, 3), (3, 4), (3, 5), (3, 6)]:
    print("metaRNA is only supported on Python 2.7, 3.3, 3.4, 3.5, 3.6.")
    return False

  if not check_rna_lib_installed(**get_rna_lib_intallation_path()):
    print(
      "metaRNA requires Vienna RNA package. Please read the docs "
      "for installation instructions.\n"
      "If you have Vienna Package, then perhaps the installer is "
      "unable to find the location. The location may be set by "
      "setting the `VIENNARNAINCLUDE` and `VIENNARNALIB` environment "
      "variables."
    )
    return False
  return True

if not check_sanity():
  sys.exit(1)

def extension_args():
  args = get_rna_lib_intallation_path()
  args['depends'] = [
    'metarna/pymiranda/miranda.h',
  ]
  args['sources'] = [
    'metarna/pymiranda/pymiranda.c',
    'metarna/pymiranda/scan.c',
    'metarna/pymiranda/swat.c',
    'metarna/pymiranda/thermo.c',
    'metarna/pymiranda/utils.c',
    'metarna/pymiranda/output.c',
    'metarna/pymiranda/ExpString.c',
  ]
  args['libraries'] = [
    'RNA',
    'm'
  ]
  if not platform.system() == 'Darwin':
    args['extra_compile_args'] = ['-fopenmp']
    args['extra_link_args'] = ['-lgomp']
  return args

setup_args = {
  'name': 'metarna',
  'version': '4.0.5',
  'author': 'Prashant Sinha',
  'author_email': 'prashant@ducic.ac.in',
  'url': 'https://github.com/prashnts/metarna',
  'license': 'GPLv3',
  'description': 'Finds target sites for the miRNAs in genomic sequences.',
  'packages': ['metarna'],
  'ext_modules': [
    Extension('metarna.pymiranda', **extension_args()),
  ],
  'classifiers': [
    'Development Status :: 4 - Beta',
    'Intended Audience :: Developers',
    'Intended Audience :: Science/Research',
    'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 3',
    'Topic :: Scientific/Engineering :: Bio-Informatics',
  ]
}

setup(**setup_args)
